@@ -2,7 +2,7 @@ import numpy as np
 from scipy.interpolate import interp1d, pchip_interpolate
 from scipy.integrate import cumtrapz
 from .modulations import AmplitudeModulations, FrequencyModulations
-from .utils import sop, bloch
+from .utils import sop, pulse_propagation, transmitter
 
 
 def nextpow2(x):
@@ -14,35 +14,46 @@ eps = np.finfo(float).eps
 
 
 class Pulse:
-    """
-    A pulse object contains everything that needs to be known about a pulse.
-    """
+    """A pulse object contains everything that needs to be known about a pulse."""
 
-    def __init__(self, pulse_time, time_step=None, flip=np.pi, mwFreq=33.80,
-                 amp=None, Qcrit=None, freq=0, phase=0, type='rectangular', **kwargs):
+    def __init__(self, pulse_time, time_step=None, flip=np.pi, type='rectangular',
+                 mwFreq=33.80, amp=None, Qcrit=None, freq=0, phase=0,
+                 exciteprofile=True, **kwargs):
         """
-        :param pulse_time: float
+
+        Parameters
+        ----------
+        pulse_time : float
             Length of pulse in us
-        :param time_step: float
-            Time increment in us
-        :param flip: float
-            Pulse flip angle
-        :param amp: float
-            Pulse maximum amplitude
-        :param Qcrit: float
-            Critcal adiabtacity
-        :param freq: float np.ndarray-lile
-            Pulse frequency offset/bandwidth.
-        :param phase: float
+        time_step : float
+            Time increment of pulse shape in us. Usually defined by the AWG time step. e.g. Bruker SpinJet time_step =
+            0.000625 (625 ns)
+        flip : float
+            Pulse flip angle in radians.
+        shape : str
+            Name of pulse shape as a string with either 'Amplitude/frequency' or just 'Amplitude'.
+        mwFreq : float
+            Microwave source base frequency in GHz.
+        amp : flaot
+            Pulse maximum amplitude in MHz.
+        Qcrit : float
+            Critical adiabtacity.
+        freq : float, Array-like
+            Pulse frequency offset/bandwidth in MHz.
+        phase : float
             Pulse Phase in radians.
-        :param type: str
-            amplitude/frequency modulation
-        :param kwargs: dict
-            Additional keyword arguments for specific pulse shapes
+        exciteprofile : Array-like
+            Resonator excitation profile. Matrix with the first row corresponding to the source frequency and the
+            second corresponding to
+        **kwargs : dict
+            Additional arguments necessary for Amplitude/Frequency modulation shapes. For a list of AM/FM functions and
+            their parameters run ``
         """
 
         # Copy input args in case needed
         self.inp_kwargs = kwargs.copy()
+        self.trajectory = kwargs.get('trajectory', False)
+        self.M0 = kwargs.get('M0', [0, 0, 1])
 
         # Separate FM and AM shapes
         ntype = len(type.split('/'))
@@ -118,6 +129,9 @@ class Pulse:
         if not hasattr(self, 'IQ'):
             self._compute_IQ()
 
+        if exciteprofile:
+            self.exciteprofile()
+
     def _shape(self):
         """Calculate shape of amplitude and frequency modulations if IQ not provided by user, otherwise set the
         time domain"""
@@ -209,7 +223,9 @@ class Pulse:
             self.amp = self.flip / (2 * np.pi * np.trapz(self.amplitude_modulation, self.time))
         else:
             if self.Qcrit is None:
-                self.Qcrit = (2 / np.pi) * np.log(2 / (1 + np.cos(self.flip)))
+                with np.errstate(divide='ignore'):
+                    self.Qcrit = (2 / np.pi) * np.log(2 / (1 + np.cos(self.flip)))
+
                 self.Qcrit = np.minimum(self.Qcrit, 5)
 
             if not hasattr(self, 'profile'):
@@ -238,6 +254,10 @@ class Pulse:
         self.phase = self.phase + 2 * np.pi * np.mean(self.freq) * self.time + self.inp_phase
         self.IQ = self.amplitude_modulation * np.exp(1j * self.phase)
 
+        # If the pulse is told to be applied in the y axis
+        if self.inp_kwargs.get('axis', 'x') == 'y':
+            self.IQ = self.IQ.imag + 1j * self.IQ.real
+
     def estimate_timestep(self):
         """Estimate times step based off of Nyquist theorem"""
         if self.fm_func.__name__ == 'none':
@@ -249,7 +269,7 @@ class Pulse:
         dt = 1e-4
         tpulse = Pulse(time_step=dt, pulse_time=self.pulse_time, flip=self.flip, amp=1,
                        mwFreq=self.mwFreq, Qcrit=self.Qcrit, freq=self.freq,
-                       phase=self.inp_phase, type=self.type, **self.inp_kwargs)
+                       phase=self.inp_phase, type=self.type, exciteprofile=False, **self.inp_kwargs)
 
         if nextpow2(len(tpulse.time)) < 10:
             zf = 2 ** 10
@@ -291,6 +311,11 @@ class Pulse:
         if filename[-4:] != '.shp':
             filename += '.shp'
 
+        # Ensure shape number is acceptable
+        if not 10 <= shape_number <=1_000_000:
+            raise ValueError('Shape number must be between 10 and 6,000,000')
+
+
         # Normalize IQ
         IQ = self.IQ / self.IQ.max()
 
@@ -335,5 +360,8 @@ class Pulse:
 
         # Calculate excitation profile
         self.offsets = np.atleast_1d(self.offsets)
-        self.Mx, self.My, self.Mz = bloch(self)
+        self.M = pulse_propagation(self, M0=self.M0, trajectory=self.trajectory)
+        self.Mx, self.My, self.Mz = np.moveaxis(self.M, -1, 0)
 
+    def transmitter(self, Ain, Aout):
+        self.IQ = transmitter(self.IQ, Ain, Aout, task='compensate')
\ No newline at end of file
